// Otto Pilot
//
// A simple Win95 cron-like program.
//
// By Eric Tauck  warp@earthling.net

// Things to add:
//
// - date/time ranges (e.g. "3-5" or "3,4,5") a la cron
// - maybe step value a la cron
// - allow multiple program instances (perhaps with command line switch); and/or
//   maybe allow a different run file to be merged with the current event list
//
// Notes:
//
// - Sunday = 1 (cron allows 0)
// - January = 1 (cron allows 0)
// - invalid numbers, spelled months and weekdays are treated as "*" (probably bad)
// - does not check for invalid dates/times (like the 40th day of May)
// - does not check for invalid program names or result of running program
// - assumes Windows timer event might be a little late (but not early) and will
//   reset to top of minute if five or more seconds late; if event IS early, all
//   events will be checked again at the next top of the minute

#include "events.h"
#include "resource.k"
#include <epp.h>

const char *TITLE = "Otto Pilot";
const char *HELPFILE = "otto.hlp";

#define EXTEN "run"
const char *FILTER = "Run Files (*." EXTEN ")\000*." EXTEN "\000All Files (*.*)\000*.*\000\000";

static char HOME [256];
static char HELP [256];
static char RunSource [256];

class MainFrame : public epp_Frame
{
    public:

        int Load (const char *commands);

    private:

        void Arrange ();
        void Open ();
        void Edit ();
        void Redraw ();
        void ResetTimer ();

        void OnClose ();
        void OnCommand (int, int, HWND);
        LRESULT OnCreate (CREATESTRUCT *);
        void OnSize (int, int, int);
        void OnTimer (UINT);

        epp_Menu menu;
        epp_Timer timer;
        epp_ListBox list;
        epp_Font font;
        Events events;
        BOOL sync;

    DECLARE_HANDLER
};

int MainFrame::Load (const char *commands)
{
    int ret;

    ret =  events.Load (commands);

    if (ret) {
        MessageBox ("Error loading run file!", TITLE, MB_OK | MB_ICONEXCLAMATION);
        ShowWindow (SW_NORMAL);
    }

    Redraw ();
    return ret;
}

void MainFrame::Arrange ()
{
    RECT rect;

    GetClientRect (&rect);
    list.MoveWindow (rect);
}

void MainFrame::Redraw ()
{
    Event event;
    char buf [512];
    char buf2 [256];
    int count;

    list.ResetContent ();

    for (count = 0; count < events.Count (); count ++) {
        event = events [count];
        if (event.minute < 0) strcpy (buf2, "--"); else wsprintf (buf2, "%02i", event.minute);
        strcpy (buf, buf2);
        if (event.hour < 0) strcpy (buf2, "--"); else wsprintf (buf2, "%02i", event.hour);
        strcat (strcat(buf, " "), buf2);
        if (event.day < 0) strcpy (buf2, "--"); else wsprintf (buf2, "%02i", event.day);
        strcat (strcat(buf, " "), buf2);
        if (event.month < 0) strcpy (buf2, "--"); else wsprintf (buf2, "%02i", event.month);
        strcat (strcat(buf, " "), buf2);
        if (event.weekday < 0) strcpy (buf2, "--"); else wsprintf (buf2, "%02i", event.weekday + 1);
        strcat (strcat(buf, " "), buf2);

        if (event.lastrun.wYear > 0)
            wsprintf (buf2, " {%02i/%02i/%02i %02i:%02i} ", event.lastrun.wMonth, event.lastrun.wDay, event.lastrun.wYear % 100, event.lastrun.wHour, event.lastrun.wMinute);
        else
            strcpy (buf2, " {-not-executed-} ");
        strcat (buf, buf2);

        strcat (buf, event.exec);
        list.AddString (buf);
    }
}

void MainFrame::Open ()
{
    epp_OpenDialog dialog;

    dialog.Create (OFN_HIDEREADONLY | OFN_EXPLORER, 0, "Open File", NULL, NULL, EXTEN, FILTER, 1, *this);

    if (dialog.OK ()) {
        strcpy (RunSource, dialog);
        Load (RunSource);
    }
}

void MainFrame::Edit ()
{
    epp_Process process;
    char buf [512];

    events.Remove ();
    Redraw ();

    wsprintf (buf, "notepad.exe %s", RunSource);

    process.Create (buf);
    process.Close ();
}

void MainFrame::ResetTimer ()
{
    SYSTEMTIME time;

    GetLocalTime (&time);

    timer.Close ();
    timer.Create ((60 - time.wSecond) * 1000, *this);
    sync = TRUE;
}

void MainFrame::OnClose ()
{
    if (MessageBox ("Closing the window only hides Otto Pilot.", TITLE, MB_OKCANCEL | MB_ICONINFORMATION) == IDOK)
        ShowWindow (SW_HIDE);
}

void MainFrame::OnCommand (int, int id, HWND)
{
    switch (id) {
        case IDM_OPEN:
            Open ();
            break;
        case IDM_EDIT:
            Edit ();
            break;
        case IDM_HIDE:
            ShowWindow (SW_HIDE);
            break;
        case IDM_QUIT:
            if (MessageBox ("Unload Otto Pilot?", TITLE, MB_YESNO | MB_ICONQUESTION) == IDYES) Close ();
            break;
        case IDM_HELP:
            WinHelp (HELP, HELP_FINDER);
            break;
    }
}

LRESULT MainFrame::OnCreate (CREATESTRUCT *)
{
    RECT rect;

    GetClientRect (&rect);

    menu.LoadMenu (IDR_MAINMENU);
    SetMenu (menu);

    font.Create (PointToPixel (10), "Courier New");
    list.Create (LBS_HASSTRINGS | WS_VISIBLE, rect, *this, 0);
    list.SetFont (font);

    ResetTimer ();
    return 0;
}

void MainFrame::OnSize (int, int, int)
{
    Arrange ();
}

void MainFrame::OnTimer (UINT)
{
    SYSTEMTIME time;

    GetLocalTime (&time);
    if (events.Check (time))
        Redraw ();

    if (sync) {
        timer.Close ();
        timer.Create (60000, *this);
        sync = FALSE;
    } else
        if (time.wSecond > 5)
            ResetTimer ();
}

BEGIN_HANDLER (MainFrame, epp_Frame)
    ON_WM_TIMER
    ON_WM_SIZE
    ON_WM_COMMAND
    ON_WM_CLOSE
    ON_WM_CREATE
END_HANDLER

int epp_Main ()
{
    HWND prev;

    prev = FindWindow (NULL, TITLE);

    if (prev) {
        ShowWindow (prev, SW_NORMAL);
        SetForegroundWindow (prev);
    } else {
        MainFrame frame;

        strcpy (RunSource, EPP_COMMANDLINE);

        frame.Create (TITLE);
        frame.ShowWindow ((strlen (RunSource))? SW_HIDE : SW_NORMAL);
        frame.UpdateWindow ();

        GetHomeDirectory (HOME);
        wsprintf (HELP, "%s%s", HOME, HELPFILE);

        if (strlen (RunSource)) frame.Load (RunSource);

        epp_App app;
        app.Run ();
    }

    return 0;
}